"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.raw = void 0;
const path_1 = __importDefault(require("path"));
const file_loader_1 = __importDefault(require("file-loader"));
const loader_utils_1 = __importDefault(require("loader-utils"));
const mime_1 = __importDefault(require("mime"));
const replace_ext_1 = __importDefault(require("replace-ext"));
const schema_utils_1 = __importDefault(require("schema-utils"));
const sharp_1 = __importDefault(require("sharp"));
const options_json_1 = __importDefault(require("./options.json"));
exports.raw = true;
function default_1(content, sourceMap) {
    const callback = this.async();
    const options = loader_utils_1.default.getOptions(this);
    const queryObject = this.resourceQuery
        ? loader_utils_1.default.parseQuery(this.resourceQuery)
        : undefined;
    const fullOptions = { ...options, ...queryObject };
    schema_utils_1.default(options_json_1.default, fullOptions, {
        name: "Image Resize Loader",
        baseDataPath: "options",
    });
    const fullOptionsWithDefaults = {
        format: getFormat(this.resourcePath),
        scaleUp: false,
        ...fullOptions,
    };
    processImage(content, fullOptionsWithDefaults)
        .then((result) => {
        const fileLoaderContext = {
            ...this,
            resourcePath: replaceExtension(this.resourcePath, fullOptionsWithDefaults.format),
            query: fullOptionsWithDefaults.fileLoaderOptions,
        };
        const fileLoaderResult = file_loader_1.default.call(fileLoaderContext, result, sourceMap);
        callback === null || callback === void 0 ? void 0 : callback(null, fileLoaderResult);
    })
        .catch((e) => {
        throw e;
    });
}
exports.default = default_1;
async function processImage(content, { width, height, fit, position, background, scale, format, quality, scaleUp, sharpOptions, }) {
    let sharpImage = sharp_1.default(Buffer.from(content));
    const { height: imageHeight, width: imageWidth, } = await sharpImage.metadata();
    const normalizedImageHeight = imageHeight !== null && imageHeight !== void 0 ? imageHeight : Number.MAX_VALUE;
    const normalizedImageWidth = imageWidth !== null && imageWidth !== void 0 ? imageWidth : Number.MAX_VALUE;
    const normalizedResultHeight = height !== null && height !== void 0 ? height : 0;
    const normalizedResultWidth = width !== null && width !== void 0 ? width : 0;
    if (scale) {
        sharpImage = sharpImage.resize({
            width: Math.round(normalizedImageWidth * scale),
            ...sharpOptions === null || sharpOptions === void 0 ? void 0 : sharpOptions.resize,
        });
    }
    else if (scaleUp ||
        (normalizedResultHeight <= normalizedImageHeight &&
            normalizedResultWidth <= normalizedImageWidth))
        sharpImage = sharpImage.resize({
            width,
            height,
            fit,
            position,
            background,
            ...sharpOptions === null || sharpOptions === void 0 ? void 0 : sharpOptions.resize,
        });
    if (format)
        sharpImage = sharpImage[format]({ quality, ...sharpOptions === null || sharpOptions === void 0 ? void 0 : sharpOptions[format] });
    return await sharpImage.toBuffer();
}
function replaceExtension(resourcePath, format) {
    if (!format)
        return resourcePath;
    if (mime_1.default.getType(format) === mime_1.default.getType(path_1.default.extname(resourcePath)))
        return resourcePath;
    return replace_ext_1.default(resourcePath, `.${format}`);
}
function getFormat(resourcePath) {
    const type = mime_1.default.getType(resourcePath);
    switch (type) {
        case "image/jpeg":
            return "jpeg";
        case "image/png":
            return "png";
        case "image/webp":
            return "webp";
        case "image/tiff":
            return "tiff";
    }
}
